#include "eeprom.h"
#include <util/delay.h>
#include <avr/io.h>

#define PULL_SDA EEPROM_DDR |= (1 << EEPROM_SDA)
#define PULL_SCL EEPROM_DDR |= (1 << EEPROM_SCL)
#define RELEASE_SDA EEPROM_DDR &= ~(1 << EEPROM_SDA)
#define RELEASE_SCL EEPROM_DDR &= ~(1 << EEPROM_SCL)

#define SDA_IS_LOW ((EEPROM_PIN & (1 << EEPROM_SDA)) == 0)
#define SCL_IS_LOW ((EEPROM_PIN & (1 << EEPROM_SCL)) == 0)

#define I2C_DELAY _delay_us(5)

#define I2C_START {RELEASE_SCL; I2C_DELAY; PULL_SDA; I2C_DELAY;}
#define I2C_STOP {I2C_DELAY; PULL_SCL; I2C_DELAY; RELEASE_SCL;  I2C_DELAY; RELEASE_SDA; I2C_DELAY;}

#define EEPROM_ADDRESS 0xA0

uint8_t i2c_write(uint8_t byte)
{
	uint8_t i;
	for (i = 0; i < 8; i++) {
		PULL_SCL;
		if (byte & 0x80) RELEASE_SDA;
		else PULL_SDA;
		I2C_DELAY;
		RELEASE_SCL;
		while (SCL_IS_LOW);
		I2C_DELAY;
		byte <<= 1;
	}
	PULL_SCL;
	RELEASE_SDA;
	I2C_DELAY;
	uint8_t ack = EEPROM_PIN & (1 << EEPROM_SDA);
	RELEASE_SCL;
	while (SCL_IS_LOW);
	I2C_DELAY;
	return ack;
}

uint8_t i2c_read(uint8_t ack)
{
	uint8_t ret = 0;
	uint8_t i;
	for (i = 0; i < 8; i++) {
		ret <<= 1;
		PULL_SCL;
		RELEASE_SDA;
		I2C_DELAY;
		if (!SDA_IS_LOW) ret |= 1;
		RELEASE_SCL;
		while (SCL_IS_LOW);
		I2C_DELAY;
	}
	PULL_SCL;
	I2C_DELAY;
	if (ack) PULL_SDA;
	else RELEASE_SDA;
	RELEASE_SCL;
	while (SCL_IS_LOW);
	I2C_DELAY;
	return ret;
}


uint8_t eeprom_read(uint16_t address, void* buf, uint16_t count)
{
	uint8_t *b = buf;
	I2C_START;
	uint8_t ack = i2c_write(EEPROM_ADDRESS);
	if (ack) {
		I2C_STOP;
		return ack;
	}
	ack = i2c_write(address >> 8);
	if (ack) {
		I2C_STOP;
		return ack;
	}
	ack = i2c_write(address & 0xFF);
	if (ack) {
		I2C_STOP;
		return ack;
	}
	I2C_STOP;
	I2C_START;
	ack = i2c_write(EEPROM_ADDRESS | 1);
	if (ack) {
		I2C_STOP;
		return ack;
	}
	while (count) {
		*b = i2c_read(count > 1);
		b++;
		count--;
	}
	I2C_STOP;
	return ack;
}

uint8_t eeprom_write(uint16_t address, void* buf, uint16_t count)
{
	uint8_t *b = buf;
	uint8_t ack = 0;
	while (count) {
		PORTC ^= 2;
		uint8_t nbytes = 64 - (address & 63);
		if (nbytes > count) nbytes = count;
		uint8_t attempts = 100;
		while (attempts--) {
			I2C_START;
			ack = i2c_write(EEPROM_ADDRESS);
			if (!ack) break;
			else I2C_STOP;
		}
		if (ack) {
			return ack;
		}
		ack = i2c_write(address >> 8);
		if (ack) {
			I2C_STOP;
			return ack;
		}
		ack = i2c_write(address & 0xFF);
		if (ack) {
			I2C_STOP;
			return ack;
		}
		count -= nbytes;
		while (nbytes) {
			ack = i2c_write(*b);
			if (ack) {
				I2C_STOP;
				return ack;
			}
			b++;
			nbytes--;
		}
		I2C_STOP;
		address += nbytes;
	}
	uint8_t attempts = 100;
	while (attempts--) {
		I2C_START;
		uint8_t ack = i2c_write(EEPROM_ADDRESS);
		I2C_STOP;
		if (!ack) break;
	}
	return ack;
}

